import { URLData } from "web-utility";
import { JsxProps, VNode } from "dom-renderer";
import { ClassComponent, FC, WebCell, WebCellProps } from "web-cell";
export enum RouterMode {
    hash = "#",
    history = "/"
}
export class History {
    baseURL: string;
    delimiter: RouterMode;
    accessor path: string;
    accessor oldPath: string;
    constructor(baseURL?: string, delimiter?: RouterMode);
    protected restore: () => void;
    push(path?: string): string;
    static dataOf(path: string): URLData<unknown>;
    match(pattern: string, path?: string): {
        [key: string]: string;
    };
    static getTitle(root: HTMLElement): string;
    titleOf(path?: string): string;
    handleLink(event: Event, link: HTMLAnchorElement): void;
    handleForm: (event: Event, form: HTMLFormElement) => void;
}
export type IncludeText<Raw extends string, Sub extends string> = {
    [K in Raw]: K extends `${string}${Sub}${string}` ? K : never;
}[Raw];
export interface PageProps extends JsxProps<HTMLElement> {
    path: string;
    history: History;
    [key: string]: any;
}
export interface CellRouteProps extends WebCellProps {
    history?: History;
    path: string;
    component: FC<PageProps> | ClassComponent;
}
export interface CellRoute extends WebCell {
}
export class CellRoute extends HTMLElement implements WebCell {
    props: CellRouteProps;
    accessor history: History | undefined;
    accessor path: string;
    component: CellRouteProps['component'];
    get matched(): {
        [key: string]: string;
    };
    connectedCallback(): void;
    render(): VNode;
}
export interface RouterOptions {
    mode?: keyof typeof RouterMode;
    basePath?: string;
}
export interface LinkProps extends WebCellProps<HTMLAnchorElement> {
    to: string;
}
export type FormProps = JsxProps<HTMLFormElement>;
export function createRouter({ mode, basePath, ...scopeProps }?: RouterOptions): {
    Route: ({ path, ...props }: CellRouteProps) => VNode;
    Link: ({ to, children, ...props }: LinkProps) => VNode;
    Form: ({ action, children, ...props }: FormProps) => VNode;
};

//# sourceMappingURL=index.d.ts.map
